import requests

from ..config import BOURSIKA_API_BASE_URL
from ..core import PredictorSystemBase


class BoursikaPredictorSystem(PredictorSystemBase):
    def publish_event(self, event_name: str, payload: dict) -> None:
        endpoint = f'{BOURSIKA_API_BASE_URL}/predictors/{self.client.slug}/events/{event_name}/'

        try:
            response = requests.post(endpoint, json={
                'payload': payload,
            }, headers={
                'ApiKey': self.client.api_key,
            }, timeout=10)

            response.raise_for_status()

            return response.json().get('result')
        except requests.exceptions.HTTPError as e:
            raise Exception(e.response.json().get('message'))



import time
from abc import ABC, abstractmethod
from typing import ClassVar

from .action_system import ActionSystemBase
from .event_system import EventSystemBase, get_event_listener_decorator
from .predictor_system import PredictorSystemBase
from .service_proxy import ServiceActionProxyFactory
from .service_types import ServiceTypes


class BasicClientBase(ABC):
    _action_system_class: ClassVar[ActionSystemBase] = None
    _event_system_class: ClassVar[EventSystemBase] = None
    _predictor_system_class: ClassVar[PredictorSystemBase] = None

    def __init__(self, api_key: str):
        self.api_key = api_key

        self.is_prepared = False
        self.is_started = False

        self._action_system = self._action_system_class(client=self)
        self._event_system = self._event_system_class(client=self)
        self._predictor_system = self._predictor_system_class(client=self)

        self.event_listener = get_event_listener_decorator(event_system=self._event_system)

        self._requirements = []

    def add_requirement(self, service_type: ServiceTypes, slug: str) -> None:
        if self.is_prepared:
            raise Exception('you should add requirements before calling prepare')

        self._requirements.append({
            'service_type': service_type,
            'slug': slug,
        })

    def _check_requirement(self, service_type: ServiceTypes, slug: str) -> bool:
        for requirement in self._requirements:
            if requirement.get('service_type') == service_type and requirement.get('slug') == slug:
                return True

        return False

    @abstractmethod
    def validate_service_access(self, service_type: ServiceTypes, slug: str):
        pass

    def prepare(self) -> None:
        self.is_prepared = True

    def get_time(self) -> int:
        return int(time.time())

    def start(self) -> None:
        self._event_system.start()


class PredictorClientBase(BasicClientBase, ABC):
    def __init__(self, api_key: str, slug: str):
        super().__init__(api_key=api_key)

        self.providers = ServiceActionProxyFactory(action_system=self._action_system,
                                                   service_type=ServiceTypes.PROVIDER)
        self.predictor = self._predictor_system
        self.slug = slug

    def validate_service_access(self, service_type: ServiceTypes, slug: str):
        if not self.is_prepared:
            raise Exception('you should call prepare before calling actions or registering event listeners')

        if not self._check_requirement(service_type, slug):
            raise Exception('add all services to requirements before using them')

        if service_type == ServiceTypes.TRADER:
            raise Exception('predictors cannot access traders')


class StrategyClientBase(BasicClientBase, ABC):
    def __init__(self, api_key: str):
        super().__init__(api_key=api_key)

        self.providers = ServiceActionProxyFactory(action_system=self._action_system,
                                                   service_type=ServiceTypes.PROVIDER)
        self.traders = ServiceActionProxyFactory(action_system=self._action_system, service_type=ServiceTypes.TRADER)

    def validate_service_access(self, service_type: ServiceTypes, slug: str):
        if not self.is_prepared:
            raise Exception('you should call prepare before calling actions or registering event listeners')

        if not self._check_requirement(service_type, slug):
            raise Exception('add all services to requirements before using them')



from abc import ABC, abstractmethod


class PredictorSystemBase(ABC):
    def __init__(self, client):
        self.client = client

    @abstractmethod
    def publish_event(self, event_name: str, payload: dict) -> None:
        pass



from os import path

from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    # region core
    name="boursika",
    version="0.1.0",
    python_requires=">=3.5",
    packages=find_packages(exclude=["scripts"]),
    install_requires=[
        "certifi==2020.6.20",
        "chardet==3.0.4",
        "idna==2.10; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "requests==2.24.0",
        "six==1.15.0; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "urllib3==1.25.11; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4' and python_version < '4'",
        "websocket-client==0.57.0",
    ],
    extras_require={
        "dev": [
            "appdirs==1.4.4",
            "attrs==20.2.0; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
            "black==19.10b0; python_version >= '3.6'",
            "bleach==3.2.1; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
            "cached-property==1.5.2",
            "cerberus==1.3.2",
            "certifi==2020.6.20",
            "cffi==1.14.3",
            "chardet==3.0.4",
            "click==7.1.2; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
            "colorama==0.4.4; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
            "cryptography==3.2.1; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
            "distlib==0.3.1",
            "docutils==0.16; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
            "idna==2.10; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
            "jeepney==0.4.3; sys_platform == 'linux'",
            "keyring==21.4.0; python_version >= '3.6'",
            "orderedmultidict==1.0.1",
            "packaging==20.4; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
            "pathspec==0.8.0",
            "pep517==0.9.1",
            "pip-shims==0.5.3; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
            "pipenv-setup==3.1.1",
            "pipfile==0.0.2",
            "pkginfo==1.6.1",
            "plette[validation]==0.2.3; python_version >= '2.6' and python_version not in '3.0, 3.1, 3.2, 3.3'",
            "pycparser==2.20; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
            "pygments==2.7.2; python_version >= '3.5'",
            "pyparsing==2.4.7; python_version >= '2.6' and python_version not in '3.0, 3.1, 3.2, 3.3'",
            "python-dateutil==2.8.1; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
            "readme-renderer==28.0",
            "regex==2020.10.28",
            "requests==2.24.0",
            "requests-toolbelt==0.9.1",
            "requirementslib==1.5.14; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
            "rfc3986==1.4.0",
            "secretstorage==3.1.2; sys_platform == 'linux'",
            "six==1.15.0; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
            "toml==0.10.1",
            "tomlkit==0.7.0; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
            "tqdm==4.51.0; python_version >= '2.6' and python_version not in '3.0, 3.1, 3.2, 3.3'",
            "twine==3.2.0",
            "typed-ast==1.4.1",
            "urllib3==1.25.11; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4' and python_version < '4'",
            "vistir==0.5.2; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
            "webencodings==0.5.1",
            "wheel==0.35.1; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
        ]
    },
    dependency_links=[],
    # endregion
    # region metadata
    description="Client Library to Connect and Use Boursika",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/ut-kdd/boursika/boursika-lib/",
    author="Mahdi Sadeghi",
    author_email="mail2mahsad@gmail.com",
    keywords="boursika",
    project_urls={
        "Bug Reports": "https://gitlab.com/ut-kdd/boursika/boursika-lib/issues",
        "Source": "https://gitlab.com/ut-kdd/boursika/boursika-lib/",
    },
    # endregion
)



from abc import ABC, abstractmethod
from typing import Callable

from .service_types import ServiceTypes


class EventListenerDescriptor:
    def __init__(self, service_type: ServiceTypes, slug: str, event_name: str, fn: Callable):
        self.service_type = service_type
        self.slug = slug
        self.event_name = event_name
        self.fn = fn


class EventSystemBase(ABC):
    def __init__(self, client):
        self.client = client

        self._event_listener_descriptors = []

    def register_event_listener(self, service_type: ServiceTypes, slug: str, event_name: str, fn: Callable) -> None:
        self.client.validate_service_access(service_type, slug)
        self._event_listener_descriptors.append(
            EventListenerDescriptor(service_type=service_type, slug=slug,
                                    event_name=event_name, fn=fn)
        )

    def trigger_event(self, service_type: ServiceTypes, slug: str, event_name: str, payload: dict) -> None:
        for descriptor in self._event_listener_descriptors:
            if descriptor.service_type == service_type and descriptor.slug == slug and descriptor.event_name == event_name:
                descriptor.fn(payload=payload)

    @abstractmethod
    def start(self) -> None:
        pass


def get_event_listener_decorator(event_system: EventSystemBase) -> Callable:
    def event_listener(service_type: ServiceTypes, slug: str, event_name: str) -> Callable:
        def decorator(fn) -> Callable:
            event_system.register_event_listener(service_type=service_type, slug=slug, event_name=event_name, fn=fn)
            return fn

        return decorator

    return event_listener
